import React, { Suspense } from 'react';
import ReactDOM from 'react-dom/client';
import './index.css';
import App from './App';
import { ThemeProvider } from "./providers/ThemeProvider";
import { MobileProvider } from "./providers/MobileProvider";
import { AuthProvider } from "./contexts/AuthContext";
import { Loader } from "@/components/ui/loader";
import { initializePerformanceOptimizations, applyMobileOptimizations } from './lib/performance-optimizations';
import { QueryClientProvider } from '@tanstack/react-query';
import { queryClient } from './lib/queryClient';
import { Toaster } from "@/components/ui/toaster";
import { HybridConnectionProvider } from './providers/HybridConnectionProvider';
import { LanguageProvider } from './contexts/LanguageContext';
import { SubscriptionProvider } from './contexts/SubscriptionContext';
import * as iOS18Detector from './lib/ios18Detector';

// Use named function for better compatibility with React Fast Refresh
function AuthProviderWrapper({ children }: { children: React.ReactNode }) {
  // Use this component to ensure only one instance of AuthProvider exists
  return <AuthProvider>{children}</AuthProvider>;
}

// Define a global flag to track if auth was initialized
declare global {
  interface Window {
    __AUTH_INITIALIZED?: boolean;
    clearStartupTimeout?: () => void;
    __ANDROID_RETRY_ATTEMPTED?: boolean;
  }
}

// Import the real IOSRedirect component with named import
import { IOSRedirect } from './components/IOSRedirect';

// We don't need to define IOSRedirectProps here as it's already defined in the component file

// Apply performance optimizations immediately
initializePerformanceOptimizations();

// Apply mobile-specific optimizations if on a mobile device
if (/Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent) ||
    (navigator.userAgent.includes("Mac") && "ontouchend" in document) ||
    window.innerWidth < 768) {
  applyMobileOptimizations();
  
  // Initialize iOS-specific patches immediately on detection
  try {
    // Run iOS detection
    const iosDetection = iOS18Detector.detect18iOS();
    if (iosDetection.isIOS) {
      // Get iOS version details for logging
      const iosVersion = iOS18Detector.getIOSVersion();
      const versionString = iosVersion 
        ? `${iosVersion.major}.${iosVersion.minor}.${iosVersion.patch}` 
        : 'unknown';
      
      console.log(`iOS device detected (${versionString}), applying specialized handling`);
      
      // Set global flag for iOS detection
      (window as any).__IOS_DETECTED = true;
      
      // For ALL iOS devices, apply some session enhancements to improve reliability
      iOS18Detector.enhanceiOS18Session();
      
      // Apply iOS 18.3.1 special patches with additional flags
      if (iosDetection.isIOS18_3_1) {
        console.log("iOS 18.3.1 detected - initializing special compatibility mode");
        
        // Set global flags for special handling in other components
        (window as any).__IOS_18_3_1_DETECTED = true;
        (window as any).__FORCE_FALLBACK_MODE = true;
        
        // Set cookie to mark iOS 18.3.1 detection for server-side awareness
        document.cookie = "iOS18_3_1_detected=true; path=/; max-age=86400";
        
        // Force the hybrid connection to use fallback mode
        localStorage.setItem('peerpa.connectionMethod', 'fallback');
      }
    }
  } catch (error) {
    // Don't prevent app start if iOS detection fails
    console.error("Error during iOS detection:", error);
  }
}

// Declare global performance metrics
declare global {
  interface Window {
    __PEERPA_LOAD_METRICS: {
      [key: string]: number | undefined;
      initStart?: number;
      appImport?: number;
      appRender?: number;
      fullAppLoaded?: number;
    };
  }
}

// Initialize the metrics object
window.__PEERPA_LOAD_METRICS = {
  initStart: performance.now()
};
const startTime = performance.now();

/**
 * Root component with all required providers
 * Fast-loading implementation to avoid timeouts
 */
function Root() {
  // Record metrics
  const renderStart = performance.now();
  window.__PEERPA_LOAD_METRICS.appImport = renderStart - startTime;
  
  // IMPORTANT: Do NOT replace the entire body as it contains our loading screen!
  // Instead, we'll just check if the loading-container exists
  const loadingContainer = document.getElementById('loading-container');
  if (!loadingContainer) {
    console.error('No loading-container found in index.html, using fallback');
    // Only create a fallback if the loading container doesn't exist
    const fallbackLoader = document.createElement('div');
    fallbackLoader.id = 'fallback-loader';
    fallbackLoader.innerHTML = `
      <div style="display:flex;align-items:center;justify-content:center;height:100vh;width:100vw;">
        <div style="text-align:center;">
          <div style="border:4px solid rgba(59,130,246,0.3);border-radius:50%;border-top:4px solid #3b82f6;width:40px;height:40px;margin:0 auto;animation:spin 1s linear infinite;"></div>
          <p style="margin-top:20px;font-family:system-ui,-apple-system,sans-serif;color:#3b82f6;">Loading PeerPa...</p>
        </div>
      </div>
      <style>@keyframes spin{0%{transform:rotate(0deg)}100%{transform:rotate(360deg)}}</style>
    `;
    document.body.appendChild(fallbackLoader);
  }
  
  // Log useful startup information
  console.log("PeerPa starting in optimized fast-loading mode");
  console.log("Device details:", {
    userAgent: navigator.userAgent,
    viewportWidth: window.innerWidth,
    viewportHeight: window.innerHeight,
    pixelRatio: window.devicePixelRatio || 1
  });
  
  // Use a streamlined implementation to render the App faster
  // Global auth state tracking to prevent duplicate auth initializations
  if (!window.__AUTH_INITIALIZED) {
    console.log('First initialization of auth system');
    window.__AUTH_INITIALIZED = true;
  } else {
    console.log('WARNING: Auth initialization already happened, fixing to prevent double login');
  }

  return (
    <QueryClientProvider client={queryClient}>
      {/* Only include AuthProvider once per application */}
      <AuthProviderWrapper>
        <MobileProvider>
          <ThemeProvider>
            <LanguageProvider>
              <SubscriptionProvider>
                <HybridConnectionProvider
                  autoConnect={true}
                  debug={false}
                  reconnectInterval={5000}
                  maxReconnectAttempts={5}
                >
                  <IOSRedirect showNotifications={false}>
                    <App />
                  </IOSRedirect>
                  <Toaster />
                </HybridConnectionProvider>
              </SubscriptionProvider>
            </LanguageProvider>
          </ThemeProvider>
        </MobileProvider>
      </AuthProviderWrapper>
    </QueryClientProvider>
  );
}

// Detect if we're on a mobile device for timeout adjustment
const isMobileDevice = /Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent) ||
    (navigator.userAgent.includes("Mac") && "ontouchend" in document) ||
    window.innerWidth < 768;

// Use a longer timeout for mobile devices
const startupTimeout = isMobileDevice ? 15000 : 8000; // 15 seconds for mobile, 8 seconds for desktop
console.log(`Setting startup timeout to ${startupTimeout}ms (${isMobileDevice ? 'mobile' : 'desktop'} device)`);

// Create a timeout to detect and handle startup failures
let startupTimer: number | null = window.setTimeout(() => {
  console.error(`Application failed to fully initialize within ${startupTimeout}ms`);
  
  // For Android devices, we'll do one automatic retry after timeout
  const isAndroid = /Android/i.test(navigator.userAgent);
  if (isAndroid && !window.__ANDROID_RETRY_ATTEMPTED) {
    console.log('Android device detected, performing automatic retry...');
    window.__ANDROID_RETRY_ATTEMPTED = true;
    
    // Clear localStorage and sessionStorage
    try {
      localStorage.removeItem('peerpa.connectionMethod');
      localStorage.removeItem('peerpa.lastConnectionAttempt');
      sessionStorage.clear();
      console.log('Cleared storage for retry');
    } catch (e) {
      console.error('Failed to clear storage:', e);
    }
    
    // Reload with force cachebust
    window.location.href = window.location.href.split('?')[0] + '?retry=' + Date.now();
    return;
  }
  
  // Check if React is already mounting
  const isAppMounting = (window as any).__APP_MOUNTING === true;
  
  // Log that we're continuing with initialization
  console.log('Startup timeout reached, but continuing to allow application to initialize');
  
  // Set a global flag that can be checked to determine if the application is in a delayed startup state
  (window as any).__STARTUP_DELAYED = true;
}, startupTimeout);

// Function to clear the timeout when app successfully loads
window.clearStartupTimeout = () => {
  if (startupTimer !== null) {
    window.clearTimeout(startupTimer);
    startupTimer = null;
    console.log('Application started successfully, cleared startup timeout');
  }
};

// Import application initialization service
import { initializeApp } from './services/AppInitializationService';
import { initializeAndroidFixes } from './lib/androidSpecificFixes';

// Listen for successful app mount to clean up any loading UI
document.addEventListener('app-mounted', () => {
  console.log('App mounted event received - cleaning up loading UI');
  if (typeof (window as any).cleanupLoadingUI === 'function') {
    (window as any).cleanupLoadingUI();
  }
});

// Utility function to clean up loading UI
function cleanupLoadingUI() {
  try {
    // Hide the loading screen from index.html
    const loadingContainer = document.getElementById('loading-container');
    if (loadingContainer) {
      loadingContainer.style.opacity = '0';
      loadingContainer.style.transition = 'opacity 0.5s ease-out';
      setTimeout(() => {
        if (loadingContainer && loadingContainer.parentNode) {
          loadingContainer.parentNode.removeChild(loadingContainer);
          console.log('Loading UI successfully removed');
        }
      }, 500);
    }
  } catch (error) {
    console.error('Error cleaning up loading UI:', error);
  }
}

// Expose cleanup function globally for use
(window as any).cleanupLoadingUI = cleanupLoadingUI;

// Mount the application
let rootElement = document.getElementById('root');

// If root element doesn't exist, create it (especially for mobile devices)
if (!rootElement) {
  console.log('Root element not found, creating it dynamically');
  rootElement = document.createElement('div');
  rootElement.id = 'root';
  
  // Apply styles for proper mobile display
  rootElement.style.width = '100%';
  rootElement.style.height = '100%';
  rootElement.style.position = 'fixed';
  rootElement.style.top = '0';
  rootElement.style.left = '0';
  rootElement.style.zIndex = '999';
  rootElement.style.overflow = 'auto';
  rootElement.style.backgroundColor = '#fff';
  
  // Check if this is iOS and add specific classes
  if (/iPhone|iPad|iPod/i.test(navigator.userAgent)) {
    rootElement.classList.add('ios-device');
    // Add iOS-specific meta viewport if needed
    if (!document.querySelector('meta[name="viewport"]')) {
      const viewportMeta = document.createElement('meta');
      viewportMeta.name = 'viewport';
      viewportMeta.content = 'width=device-width, initial-scale=1, maximum-scale=1, user-scalable=no, viewport-fit=cover';
      document.head.appendChild(viewportMeta);
    }
  }
  
  document.body.appendChild(rootElement);
  
  // Also ensure body and html have proper styles for mobile
  document.body.style.margin = '0';
  document.body.style.padding = '0';
  document.body.style.height = '100%';
  document.body.style.width = '100%';
  document.body.style.overflow = 'hidden';
  document.documentElement.style.height = '100%';
  document.documentElement.style.width = '100%';
}

// Add warning handler for unhandled promise rejections
window.addEventListener('unhandledrejection', (event) => {
  console.error('Unhandled Promise Rejection:', event.reason);
});

// SIMPLIFIED INITIALIZATION APPROACH
// Remove complexity and async initialization to reduce startup issues
(() => {
  // Check if we should skip the normal entry point for mobile
  if ((window as any).__SKIP_NORMAL_ENTRY) {
    console.log('Skipping normal entry point - using mobile-specific entry');
    return;
  }
  
  try {
    console.log('Starting simplified app initialization...');
    
    // Clean up any loading UI
    try {
      cleanupLoadingUI();
    } catch (cleanupError) {
      console.error('Error during initial cleanup:', cleanupError);
    }
    
    // Apply device-specific fixes synchronously
    if (/Android/i.test(navigator.userAgent)) {
      console.log('Android device detected, applying fixes...');
      initializeAndroidFixes();
    }
    
    // Create a new React root directly with strict error handling
    try {
      console.log('Creating React root and rendering app...');
      
      // Force cleanup of the root element first to prevent mounting issues
      if (rootElement) {
        // Keep the element but remove any content
        while (rootElement.firstChild) {
          rootElement.removeChild(rootElement.firstChild);
        }
      }
      
      // Create and store the root
      const reactRoot = ReactDOM.createRoot(rootElement);
      reactRoot.render(
        <React.StrictMode>
          <QueryClientProvider client={queryClient}>
            <AuthProviderWrapper>
              <MobileProvider>
                <ThemeProvider>
                  <LanguageProvider>
                    <SubscriptionProvider>
                      <HybridConnectionProvider
                        autoConnect={true}
                        debug={false}
                        reconnectInterval={5000}
                        maxReconnectAttempts={5}
                      >
                        <IOSRedirect showNotifications={false}>
                          <App />
                        </IOSRedirect>
                        <Toaster />
                      </HybridConnectionProvider>
                    </SubscriptionProvider>
                  </LanguageProvider>
                </ThemeProvider>
              </MobileProvider>
            </AuthProviderWrapper>
          </QueryClientProvider>
        </React.StrictMode>
      );
      
      // Mark as successful
      (window as any).__REACT_ROOT = reactRoot;
      
      // Clear startup timeout now that we've rendered successfully
      if (window.clearStartupTimeout) {
        window.clearStartupTimeout();
      }
      
      // Dispatch app-mounted event
      const appMountedEvent = new Event('app-mounted', { bubbles: true });
      document.dispatchEvent(appMountedEvent);
      
      console.log('React app mounted successfully');
    } catch (renderError) {
      console.error('Error during React rendering:', renderError);
      
      // Fallback rendering with minimal providers on error
      try {
        console.log('Attempting minimal fallback rendering...');
        const fallbackRoot = ReactDOM.createRoot(rootElement);
        fallbackRoot.render(
          <MobileProvider>
            <ThemeProvider>
              <App />
            </ThemeProvider>
          </MobileProvider>
        );
      } catch (fallbackError) {
        console.error('Fatal: All rendering approaches failed:', fallbackError);
        throw fallbackError;
      }
    }
  } catch (error: any) {
    console.error('Critical rendering error:', error);
    
    // Display error message to user in case of catastrophic failure
    rootElement.innerHTML = `
      <div style="padding: 20px; text-align: center;">
        <h2>Application Error</h2>
        <p>The application encountered a critical error during startup.</p>
        <button onclick="window.location.reload()">Retry</button>
        <p><small>Error: ${error?.message || 'Unknown error'}</small></p>
      </div>
    `;
  }
})();

// We already have a global declaration for clearStartupTimeout above